##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import os
from string import strip, split, find, lower, join

import sys


import ingresdbi
import Globals

from Shared.DC.ZRDB.THUNK import THUNKED_TM as TM



class FetchAll:
    def __init__(self, cursor):
        self.__cursor = cursor
    def __getitem__(self, index):
        result = self.__cursor.fetchone()
        if result is not None:
            return result
        else:
            raise IndexError


datatype_mapping = { 'int' : 'i', 'string' : 's', 'Ingres date/time' : 'd', 'float' : 'n' }


class DB(TM):

    opened=''
    db=None
    in_transaction='False'


    def __init__(self,connection):
        self.connection=connection
	
	info = split(connection)
	
	if len(info)==3:
            self.database = str(info[0])
	    if str(info[1]) == 'local':
	        self.vnode="("+str(info[1])+")"
	    else:
	        self.vnode=str(info[1])
	    self.trace=(int(info[2]),None)
	elif len(info)==2:
            self.database = str(info[0])
	    if str(info[1]) == 'local':
	        self.vnode="("+str(info[1])+")"
	    else:
	        self.vnode=str(info[1])
	    self.trace=(0,None)    
        elif len(info)==1:
            self.database = str(info[0])
	    self.vnode="(local)"
	    self.trace=(0,None)
	
        self.open()

    # begin/start transaction
    def _begin(self):
        self.open()


    # abort (rollback) transaction
    def _abort(self):
        if self.db is not None:
            if self.in_transaction=='True':
                self.db.rollback()
                self.in_transaction='False'

    # finish (commit) transaction
    def _finish(self):
        #print 'finish'
        if self.db is not None:
            if self.in_transaction=='True':
                #print 'commit'
                self.db.commit()
                self.in_transaction='False'
		#print 'finishlast'

    def tables(self,*args,**kw):
        table_list_sql = "SELECT table_name, table_type FROM iitables WHERE table_owner != '$ingres' and table_name not like 'ii%' and (table_type='T' or table_type='V')"
        items, rows = self.query(table_list_sql)
        
        table_type_map = { 'T': 'table', 'V':'view' }
        result = []
        for tmp_row in rows:
            result.append({'TABLE_NAME':tmp_row[0], 
                           'TABLE_TYPE':table_type_map[tmp_row[1]]})
        return result


    # Not yet completed/used - for column browsing
    # intended to replace columns
    def columns_wip(self, table_name):

        if self.db is None: self.open()
        XYZ = {
              'Name': 'col1', 'Type': 's', 'Precision': 0,
              'Scale': 0, 'Nullable': 'with Null'
              }
        return XYZ

    def columns(self, table_name):
        '''
        if self.db is None: self.open()
        SELECT column_name, column_datatype, column_length,
               column_scale, column_nulls
        FROM iicolumns
        WHERE table_name = 'table_name'
          AND table_owner = USER
        '''
        # Need to check for columns on tables and then deal with views
        # should be able to query iicolumns for views too
        # quick hack
        str = "***** column browsing on objects is not yet implemented *****"
        return str

    # open connection to database
    def open(self):
        

        if self.db is None: 

            self.db=ingresdbi.connect(database=str(self.database),vnode=str(self.vnode),trace=self.trace)

            self.cursor=self.db.cursor()
        self.opened='true'

    # close database connection
    def close(self):

        if self.db is not None: 
            self.db.close()
        self.db=None
        del self.opened

    # Runs SQL query (which may be a select, update, delete, etc.)
    # opens a database connection if required and re-uses the opened
    # cursor, i.e. only one query at a time possible.
    def query(self,query_string=None, max_rows=None):
        #print '1'
        if self.db is None: self.open()
        self._register()
	#print '2'
        queries=filter(None, map(strip,split(query_string, '\0')))

        if not queries: raise 'Query Error', 'empty query'
        desc=None
        result=[]
        #print 'CMC query queries:' + str(queries)
        for qs in queries:

            if find(lower(qs),'execute procedure') > -1:
	            #print 'exe'
	    #	    self.cursor.execute('set autocommit on')
	    	    exe_res = self.cursor.execute(qs)

		    return (),()
		    #self.cursor.execute('set autocommit off')

	    else:
	        self.cursor.execute('set autocommit on')
	    	exe_res = self.cursor.execute(qs)

#	    	    print 'proc'
#		    qproc = qs.replace('select','execute procedure')
#		    exe_res = self.cursor.execute(qproc)
#		    return (),()
	    #if find(lower(qs),'update') > -1 or find(lower(qs),'insert') > -1 or  find(lower(qs),'delete') > -1:
                #self.cursor.execute('set autocommit on')
	        #exe_res = self.cursor.execute(qs)
		#return (),()
		#self.cursor.execute('set autocommit off')

            #else:
	    #    if find(lower(qs),'select') > -1 and find(lower(qs),'from') > -1:
	    #	    self.cursor.execute('set autocommit on')
	    #	    exe_res = self.cursor.execute(qs)
		    #self.cursor.execute('set autocommit off')

	    #	else:
#	    	    print 'proc'
#		    qproc = qs.replace('select','execute procedure')
#		    exe_res = self.cursor.execute(qproc)
#		    return (),()
		    #try:
		    #    qproc = qs.replace('select','execute procedure')
		    #	exe_res = self.cursor.execute(qproc)

		    #except:
		    #    pass

		        #qrpproc = qs.replace('select','call')
			#qrpproc = '{'+qrpproc+'}'

			#exe_res = self.cursor.execute(qrpproc)

            #import pdb
	    #pdb.set_trace()
            self.in_transaction='True'
            if exe_res is None:
                if desc is None:
                    desc=self.cursor.description

                    if desc != [] and desc is not None:
                        if max_rows:
                            #print 'CMC query max_rows:' + str(max_rows)
                            if max_rows==1: result=(self.cursor.fetchone(),)
                            else:
			        result=self.cursor.fetchall()
                        else:

			    result=self.cursor.fetchall()


        if desc is None: return (),()

        # So we have some results, now we map the datatypes
        # Use the maping dictionary and default to string for unknown datatypes
        #
        items=[]
        for name, type, display_size, internal_size, precision, scale, null_ok in desc:
            try:
                ZDA_type = datatype_mapping[type]
            except KeyError:
                ZDA_type = 's'

            if ZDA_type == 's':
                ZDA_width = internal_size
            else:
                ZDA_width = None

            items.append({
                'name': name,
                'type': ZDA_type,
                'width': ZDA_width,
                'null': null_ok,
                })


        return items, result
